/*
 * INNOV-2: Build a Better Web Interface Using AJAX
 *
 * ajaxcustdtl.p
 *
 * This program generates an xml response to an AJAX request for 
 * customer detail
 *
 * Note that the xml responses must be sent with a text/xml content type.
 * If you are using WebSpeed, then you should set the content type with the
 * OutputContentType procedure provided by Progress.
 */

{cgidefs.i}

Define variable formCustNum     as character no-undo.
Define variable formNavType     as character no-undo.
Define variable intCustNum      as integer no-undo.
Define variable ValidCustomer   as logical no-undo.


/*
 *  This function is similar to the url-encode WebSpeed function, but it
 *  specificly targets the unsafe characters for xml 1.0 as described in
 *  http://www.w3.org/TR/2004/REC-xml-20040204/#syntax
 */
FUNCTION xml-encode RETURNS CHARACTER
  (INPUT ipCharData AS CHARACTER):

  Define variable xml-unsafe  as character no-undo initial "<>&":U.
  Define variable Idx         as integer no-undo.
  Define variable AsciiValue  as integer no-undo.
  Define variable HexChars    as character no-undo initial "0123456789ABCDEF":U.

  /* Don't bother with blank or unknown  */
  if length( ipCharData ) = 0 or ipCharData = ? then
    return "":U.

  /* Loop through entire input string */
  assign Idx = 0.
  do while true:
    assign Idx = Idx + 1
           /* ASCII value of character using single byte codepage */
           AsciiValue = asc( substring( ipCharData, Idx, 1, "RAW":U ),
                             "1252", "1252" ).

    if AsciiValue le 31 or AsciiValue ge 127
       or index( xml-unsafe, chr( AsciiValue ) ) gt 0 then
    do:
      /* Replace character with %hh hexidecimal triplet */
      substring( ipCharData, Idx, 1, "RAW":U ) =
        "%":U +
        substring( HexChars, integer( truncate( AsciiValue / 16, 0 ) ) + 1,
                   1, "RAW":U) + /* high */
        substring( HexChars, AsciiValue modulo 16 + 1, 1, "RAW":U ). /* low */
      assign Idx = Idx + 2.   /* skip over hex triplet just inserted */
    end.
    if Idx = length(ipCharData,"RAW") then leave.
  end.

  return ipCharData.

END FUNCTION.  /**  xml-encode  **/


assign formNavType = get-value("nav")
       formCustNum = get-value("cnum")
       intCustNum  = ?.

/*
 *  If no navigation is requested, validate and return the requested customer.
 */
if formNavType eq ? or formNavType eq ""
   or lookup( formNavType, "f,p,n,l" ) eq 0 then
do:
  /**  first, check to see if we even have an integer customer number  **/
  assign intCustNum = integer( formCustNum ) NO-ERROR.

  if intCustNum eq ? then
    ValidCustomer = no.
  else
    ValidCustomer = can-find( Customer where Customer.CustNum eq intCustNum ).
end.

else if formNavType eq "f" then  /**  first customer  **/
do:
  find first Customer no-lock no-error.
  assign intCustNum    = Customer.CustNum
         ValidCustomer = yes.
end.
else if formNavType eq "l" then  /**  last customer  **/
do:
  find last Customer no-lock no-error.
  assign intCustNum    = Customer.CustNum
         ValidCustomer = yes.
end.
else if formNavType eq "p" then  /**  previous customer  **/
do:
  assign intCustNum = integer( formCustNum ) NO-ERROR.

  if intCustNum ne ? then
    find Customer where Customer.CustNum eq intCustNum no-lock no-error.
  if available Customer then
    find prev Customer no-lock no-error.

  /**  if no customer is found, return the first customer  **/
  if not available Customer then
    find first Customer no-lock no-error.

  assign intCustNum    = Customer.CustNum
         ValidCustomer = yes.
end.
else if formNavType eq "n" then  /**  next customer  **/
do:
  assign intCustNum = integer( formCustNum ) NO-ERROR.

  if intCustNum ne ? then
    find Customer where Customer.CustNum eq intCustNum no-lock no-error.
  if available Customer then
    find next Customer no-lock no-error.

  /**  if no customer is found, return the last customer  **/
  if not available Customer then
    find last Customer no-lock no-error.

  assign intCustNum    = Customer.CustNum
         ValidCustomer = yes.
end.


{&OUT}
  '<?xml version="1.0" encoding="UTF-8" standalone="yes"?>' skip
  '<customer>' skip.

if not ValidCustomer then
  {&OUT}
    '  <valid>No</valid>' skip.

else do:

  {&OUT}
    '  <valid>Yes</valid>' skip.

  for first Customer fields ( CustNum Name Address Address2 City State
                              PostalCode Contact Phone Fax )
            where Customer.CustNum eq intCustNum
            no-lock:
     {&OUT}
       '  <custnum>' Customer.CustNum '</custnum>' skip
       '  <name>' xml-encode( Customer.Name ) '</name>' skip
       '  <addr1>' xml-encode( Customer.Address ) '</addr1>' skip
       '  <addr2>' xml-encode( Customer.Address2 ) '</addr2>' skip
       '  <city>' xml-encode( Customer.City ) '</city>' skip
       '  <st>' xml-encode( Customer.State ) '</st>' skip
       '  <zip>' xml-encode( Customer.PostalCode ) '</zip>' skip
       '  <contact>' xml-encode( Customer.Contact ) '</contact>' skip
       '  <phone>' xml-encode( Customer.Phone ) '</phone>' skip
       '  <fax>' xml-encode( Customer.Fax ) '</fax>' skip
       skip
       .
  end.  /**  for first Customer  **/
end.  /**  valid Customer  **/

{&OUT}
  '</customer>' skip.

