/*
 * INNOV-2: Build a Better Web Interface Using AJAX
 *
 * ajaxsesdet.p
 *
 * This program generates an xml response to an AJAX request for the
 * details of a breakout session.
 *
 * Note that the xml responses must be sent with a text/xml content type.
 * If you are using WebSpeed, then you should set the content type with the
 * OutputContentType procedure provided by Progress.
 */

{cgidefs.i}

Define variable SessionID    as  character no-undo.


/*
 *  This function is similar to the url-encode WebSpeed function, but it
 *  specificly targets the unsafe characters for xml 1.0 as described in
 *  http://www.w3.org/TR/2004/REC-xml-20040204/#syntax
 */
FUNCTION xml-encode RETURNS CHARACTER
  (INPUT ipCharData AS CHARACTER):

  Define variable xml-unsafe  as character no-undo initial "<>&":U.
  Define variable Idx         as integer no-undo.
  Define variable AsciiValue  as integer no-undo.
  Define variable HexChars    as character no-undo initial "0123456789ABCDEF":U.

  /* Don't bother with blank or unknown  */
  if length( ipCharData ) = 0 or ipCharData = ? then
    return "":U.

  /* Loop through entire input string */
  assign Idx = 0.
  do while true:
    assign Idx = Idx + 1
           /* ASCII value of character using single byte codepage */
           AsciiValue = asc( substring( ipCharData, Idx, 1, "RAW":U ),
                             "1252", "1252" ).

    if AsciiValue le 31 or AsciiValue ge 127
       or index( xml-unsafe, chr( AsciiValue ) ) gt 0 then
    do:
      /* Replace character with %hh hexidecimal triplet */
      substring( ipCharData, Idx, 1, "RAW":U ) =
        "%":U +
        substring( HexChars, integer( truncate( AsciiValue / 16, 0 ) ) + 1,
                   1, "RAW":U) + /* high */
        substring( HexChars, AsciiValue modulo 16 + 1, 1, "RAW":U ). /* low */
      assign Idx = Idx + 2.   /* skip over hex triplet just inserted */
    end.
    if Idx = length(ipCharData,"RAW") then leave.
  end.

  return ipCharData.

END FUNCTION.  /**  xml-encode  **/


assign SessionID = get-value("sid").

{&OUT}
  '<?xml version="1.0" encoding="UTF-8" standalone="yes"?>' skip
  '<session-detail>' skip.

for first class where class.classid eq SessionID no-lock:

  {&OUT}
    '  <sid>' class.classid '</sid>' skip
    '  <sdate>' class.classdate '</sdate>' skip
    '  <stime>' string(class.classtime, "xx:xx" ) '</stime>' skip
    '  <sname>' xml-encode( class.classname ) '</sname>' skip
    '  <speaker>' xml-encode( class.speaker ) '</speaker>' skip
    '  <sdesc>' xml-encode( class.classdesc ) '</sdesc>' skip.

end.

{&OUT}
  '</session-detail>' skip.

